/*
 * Copyright (c) 2016,2017 Daichi GOTO
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are 
 * met: 
 * 
 * 1. Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright 
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the distribution. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "command.h"

struct textset cmdtextsets[] = {
	{ "command_version", "en_", VERSION },

	{ "command_name", "en_", CMDNAME },

	{ "command_alias", "en_", ALIAS },

	{ "command_copyright", "en_", "2016,2017 ONGS Inc." },

	{ "command_comment", "ja_JP",
	  "列にHTML SELECTデータを割り当てる" },

	{ "command_comment", "en_",
	  "assign HTML SELECT data to specified columns" },

	{ "command_synopsis", "en_", 
	  _CMD(CMDNAME) " "
	  "[" _OPT("d") " " _ARG("delim") "] " 
	  "[" _OPT("1") "] " 
	  "[" _OPT("r") "] " 
	  "[" _OPT("hvD") "] [" _OPT("-") "] "
	  _ARG("N.f.K.A") "|\n" _ARG("N/M.f.K.A") " "
	  "[" _ARG("N.f.K.A") "|" _ARG("N/M.f.K.A") "|"
	  _ARG("N._attr_.a.v") "|" _ARG("N/M._attr_.a.v")
	  " " _ETC "]\n"
	  "[" _ARG("file") " " _ETC "]" },

	{ "command_description", "ja_JP", 
	  "列にHTML SELECTデータを割り当てる。\n"
	  "\n"
	  ES_BOLD("SELECT生成代入") "\n"
	  _ARG("N.f.K.A") "\t\t" _ARG("N") "列にHTML SELECTデータを割り"
	  	"当てる。HTML SELECT\n\t\tデータは" _ARG("f") "ファイルの"
		_ARG("K") "列の値をvalue属性の値に使用し、\n\t\t" 
		_ARG("f") "ファイルの" _ARG("A") "列の値をラベルとして"
		"使用し作成する。\n\t\t" 
		_ARG("A") "には複数の列を指定可能。" _ARG("N") "列の値と"
		_ARG("K") "列の値が同じ\n\t\tだった場合、該当するの"
		"option要素にはselected属性を\n\t\t追加\n"
	  _ARG("N/M.f.K.A") "\t" _ARG("N.f.K.A") "と同じ処理を" 
	  	_ARG("M") "列まで適用\n"
	  "\n"
	  "作成するSELECT要素には任意の属性を指定可能。\n"
	  "\n"
	  ES_BOLD("SELECT属性指定") "\n"
	  _ARG("N._attr_.a.v") "\t" _ARG("N") "列のSELECT要素に"
	  	_ARG("v") "の値を指定した属性" _ARG("a") "を追加\n"
	  _ARG("N/M._attr_.a.v") "\t" _ARG("N._attr_.a.v") 
	  	"と同じ処理を" _ARG("M") "列まで適用\n"
	  "\n"
	  "ファイルの指定がないか、-が指定されている場合には標準入力を"
	  "使用。" },

	{ "command_description", "en_", 
	  "Assign new generated HTML SELECT data to specified columns.\n"
	  "\n"
	  ES_BOLD("SELECT-GENERATION ASSIGNMENT") "\n"
	  _ARG("N.f.K.A") "\t\tAssign HTML SELECT data to specified "
	  	"columns.\n\t\tThe data will be generated by the value "
		"of\n\t\t" _ARG("K") "th column of the file " _ARG("f") " "
		"as values of VALUE\n\t\tattribution and the values of "
		_ARG("A") "th column of\n\t\tthe file " _ARG("f") " "
		"as LABEL data of OPTION elements.\n\t\tYou can specify "
		"one or more columns as " _ARG("A") ".\n\t\t"
		"A SELECTED attribution will be inserted into\n\t\t"
		"the OPTION element if the value of " _ARG("N")
		"th column\n\t\tand the value of " _ARG("K") "th column "
		"of the file " _ARG("f") " match.\n"
	  _ARG("N/M.f.K.A") "\tProcess " _ARG("N") "th to " _ARG("M")
		"th columns in the same way as\n\t\t" _ARG("N.f.K.A") ".\n"
	  "\n"
	  ES_BOLD("SELECT-ATTRIBUTION ASSIGNMENT") "\n"
	  _ARG("N._attr_.a.v") "\t" "Assign the attribution " _ARG("a")
	  	" that has the value " _ARG("v") " to\n\t\t"
		"the SELECT element of the " _ARG("N") "th column.\n"
	  _ARG("N/M._attr_.a.v") "\t" "Process " _ARG("N") "th to " 
	  	_ARG("M") "th columns in the same way as\n\t\t"
		_ARG("N._attr_.a.v") ".\n"
	  "\n"
	  "If " _ARG("file") " is a single dash (`-') or absent, it reads "
	  "from the\nstandard input." },

	{ "command_options", "ja_JP", 
	  _OPT("d") " " _ARG("delim") "\tHTML SELECTのラベル間区切り"
	  	"文字列を指定\n\t\t(デフォルトは空白)\n"
	  _OPT("1") "		ファイル" _ARG("f") "の1行目を無視\n"
	  _OPT("r") "		前の列の値を参照使用可能(\\1, \\2...)\n"
	  _OPT("h") "		使い方表示\n"
	  _OPT("v") "		バージョン表示\n"
	  _OPT("D") "		デバッグモード\n"
	  _OPT("-") "		オプションの終了を指定\n"
	  _ARG("file") "\t	ファイルを指定" },
	
	{ "command_options", "en_", 
	  _OPT("d") " " _ARG("delim") "\tSet the delimiter between "
	  	"HTML SELECT text\n\t\t(default is a space).\n"
	  _OPT("1") "		Ignore the first line of the file " _ARG("f") ".\n"
	  _OPT("r") "		Use references to the pre columns "
	  			"(\\1, \\2...)\n"
	  _OPT("h") "		Print the usage message.\n"
	  _OPT("v") "		Print the version.\n"
	  _OPT("D") "		Enable the debug mode.\n"
	  _OPT("-") "		Specify the end of options.\n"
	  _ARG("file") "\t	Specify the file." },
	
	{ "command_example", "en_", 
	  _P("cat vegetable.ssv")
	  _S("000001 EUROPEAN_PEAR CHINA")
	  _S("000002 FRENCH_BEAN CHINA")
	  _S("000003 JAPANESE_PARSLEY JAPAN")
	  _P("cat table.ssv")
	  _S("SELECT 000003 000001 000000")
	  _P("cat table.ssv |")
	  _P("retu_assign_htmlselect \\")
	  _P2("\t2:vegetable.ssv:1:2 \\")
	  _P2("\t2._attr_.name.vege1 \\")
	  _P2("\t4:vegetable.ssv:1:2 \\")
	  _P2("\t4._attr_.name.vage2")
_S("SELECT <select_name=\"vege1\"><option_value=\"000001\">EUROPEAN_PEAR")
_S("</option><option_value=\"000002\">FRENCH_BEAN</option><option_valu")
_S("e=\"000003\"_selected>JAPANESE_PARSLEY</option></select> 000001 <s")
_S("elect_name=\"vege2\"><option_value=\"000001\">EUROPEAN_PEAR</option>")
_S("<option_value=\"000002\">FRENCH_BEAN</option><option_value=\"000003")
_S("\">JAPANESE_PARSLEY</option></select>")
	  _P("cat table.ssv |")
	  _P("retu_assign_htmlselect \\")
	  _P2("\t-d ',' \\")
	  _P2("\t2/4:vegetable.ssv:1:1/3 \\")
	  _P2("\t4._attr_.name.vegetable")
_S("SELECT <select><option_value=\"000001\">000001,EUROPEAN_PEAR,CHINA")
_S("</option><option_value=\"000002\">000002,FRENCH_BEAN,CHINA</option")
_S("><option_value=\"000003\"_selected>000003,JAPANESE_PARSLEY,JAPAN</")
_S("option></select> <select><option_value=\"000001\"_selected>000001,")
_S("EUROPEAN_PEAR,CHINA</option><option_value=\"000002\">000002,FRENCH")
_S("_BEAN,CHINA</option><option_value=\"000003\">000003,JAPANESE_PARSL")
_S("EY,JAPAN</option></select> <select_name=\"vegetable\"><option_valu")
_S("e=\"000001\">000001,EUROPEAN_PEAR,CHINA</option><option_value=\"000")
_S("002\">000002,FRENCH_BEAN,CHINA</option><option_value=\"000003\">000")
_S("003,JAPANESE_PARSLEY,JAPAN</option></select>")
	  _P("cat table.ssv |")
	  _P("retu_assign_htmlselect -r \\")
	  _P2("\t4:vegetable.ssv:1:2 \\")
	  _P2("\t4._attr_.name.vage2 \\")
	  _P2("\t4._attr_.id.'\\1:\\2'")
_S("SELECT 000003 000001 <select_name=\"vege2\"_id=\"SELECT:000003\"><op")
_S("tion_value=\"000001\">EUROPEAN_PEAR</option><option_value=\"000002\"")
_S(">FRENCH_BEAN</option><option_value=\"000003\">JAPANESE_PARSLEY</op")
_S("tion></select>")
	  _P("") },

	TEXTSET_END
};
